#!/usr/bin/python3
"""
Cheap Pie Hardware Abstraction Layer
"""
#
# -*- coding: utf-8 -*-
## this file is part of cheap_pie, a python tool for chip validation
## author: Marco Merlin
## email: marcomerli@gmail.com

import hickle as hkl

import cheap_pie.tools.search # pylint: disable=W0611
from   cheap_pie.tools.hal2doc import hal2doc

class CpHal():
    """
    Cheap Pie Hardware Abstraction Layer
    """
    regs = []
    hif = None

    def __init__(self,regs):
        if isinstance(regs,CpHal):
            # this allows generating CpHal super-classes from a CpHal instance
            self.regs = regs.regs
        else:
            self.regs = regs
            if len(regs) > 0:
                self.hif = regs[0].hif
            else:
                print('# WARNING: no register defined!')

    def __len__(self):
        return len(self.regs)

    def __iter__(self):
        return self.regs.__iter__()

    def __next__(self):
        return self.regs.next()

    def __getitem__(self, idx):
        if isinstance(idx,int):
            return self.regs[idx]
        if isinstance(idx,str):
            return self.regs._asdict()[idx]
        assert False,'Unsupported indexing!'

    def __setitem__(self, idx, value):
        if isinstance(idx,int):
            return self.regs[idx].setreg(value)
        if isinstance(idx,str):
            return self.regs._asdict()[idx].setreg(value)
        assert False, 'Unsupported indexing!'

    def __repr__(self):
        return 'CP_HAL: %d registers' % len(self.regs) # pylint: disable=C0209

## search methods ###########################################################

    def search_bitfield(self,field,case_sensitive=False):
        """
        Search bitfields which name contains a specified string
        """
        return cheap_pie.tools.search.bitfield(self.regs,field,case_sensitive=case_sensitive) # pylint: disable=E0601

    def search_register(self,reg,case_sensitive=False):
        """
        Search registers which name contains a specified string
        """
        return cheap_pie.tools.search.register(self.regs,reg,case_sensitive=case_sensitive)

    def search_address(self,address,mask='0xFFFFFFFF'):
        """
        Search register matching a specified address
        """
        return cheap_pie.tools.search.address(self.regs,address,mask=mask)

    def to_docx(self,*args):
        """
        Generate a .docx document describing an Hardware Abstraction Layer
        """
        hal2doc(self.regs,*args)

## dump methods #############################################################
    def regs2dict(self):
        """
        Converts an hal structure into a dictionary in the form:
        dictval = {
            'reg1': val1,
            'reg2': val2,
        }
        """
        outdict = {}
        for reg,val in self.regs._asdict().items():
            outdict[reg] = val.getreg()
        return outdict

    def dump(self,fname='dump.hkl'):
        """
        Dump the value of all registers in a .hkl file.
        """
        regs_dict = self.regs2dict()
        hkl.dump(regs_dict, fname, compression='gzip')

    def dump_diff(self,f1name='dump.hkl',f2name='dump2.hkl',width = 60):
        """
        Perform a diff on two dumped .hkl files.
        """
        fmtstr = '%%%ds |%%%ds' % (width,width) # pylint: disable=C0209

        field1 = hkl.load(f1name)
        field2 = hkl.load(f2name)

        # create a header with filenames
        outstrlist = []
        for reg,val in field1.items():
            if not field2[reg] == val:
                f1regstr = self[reg].__repr__(val        ).split('\n') # pylint: disable=C2801
                f2regstr = self[reg].__repr__(field2[reg]).split('\n') # pylint: disable=C2801

                for idx in range(len(f1regstr)): # pylint: disable=C0200
                    if not f1regstr[idx]==f2regstr[idx]:
                        outstrlist.append(fmtstr % (f1regstr[idx],f2regstr[idx]))

        # print output
        if len(outstrlist) > 0:
            headerstr = fmtstr % (f1name,f2name)
            outstrlist.insert(0,headerstr)
            for line in outstrlist:
                print(line)
        else:
            print('No differences found!!!')

        return outstrlist

class CpHalSuper(CpHal):
    def super_method(self):
        print('Super Method!!!')

def test_cp_hal_to_docx():
    """
    Test Function for Cheap Pie HAL to .docx
    """
    from parsers.cp_parsers_wrapper import cp_parsers_wrapper  # pylint: disable=C0415,C0413,E0401
    from cheap_pie_core.cp_cli import cp_cli                   # pylint: disable=C0415,C0413,E0401

    prms = cp_cli(['-t','dummy','-rf','my_subblock.xml','-fmt','ipxact'])
    hal = cp_parsers_wrapper(prms)
    hal.to_docx()

def test_cp_hal(): # pylint: disable=R0914,R0915
    """
    Test Function for Cheap Pie Hardware Abstraction Layer
    """
    from ast import literal_eval                                        # pylint: disable=C0415
    from parsers.cp_parsers_wrapper import cp_parsers_wrapper # pylint: disable=C0415,E0401
    from transport.cp_dummy_transport import CpDummyTransport # pylint: disable=C0415,E0401
    from cheap_pie_core.cp_cli import cp_cli                  # pylint: disable=C0415,E0401

    print('# hal initialize')
    prms = cp_cli(['-t','dummy'])
    hal = cp_parsers_wrapper(prms,hif=CpDummyTransport())

    print('# hal test register methods...')
    print('# hal hex assignement')
    inval = "0xFFFFFFFF"
    hal.regs.ADC_ANA_CTRL.setreg(inval)
    retval = hex(hal.regs.ADC_ANA_CTRL.getreg())
    print(retval)
    assert literal_eval(inval) == literal_eval(retval)

    print('# hal decimal assignement')
    inval = 2
    hal.regs.ADC_ANA_CTRL.setreg(inval)
    retval = hal.regs.ADC_ANA_CTRL.getreg()
    assert inval == retval

    print('# hal reg representation')
    hal.regs.ADC_ANA_CTRL # pylint: disable=W0104

    print('# hal reg display')
    hal.regs.ADC_ANA_CTRL.display()

    print('# hal reg print')
    print(hal.regs.ADC_ANA_CTRL)

    print('# hal reg str')
    print(str(hal.regs.ADC_ANA_CTRL))

    print('# hal test bitfield methods...')
    print('# hal bitfield display')
    hal.regs.ADC_ANA_CTRL.bitfields.ADC_BM.display()
    print('# hal bitfield display with value')
    hal.regs.ADC_ANA_CTRL.bitfields.ADC_BM.display(2)
    print('# hal bitfield setbit')
    hal.regs.ADC_ANA_CTRL.bitfields.ADC_BM.setbit(inval)
    retval = hal.regs.ADC_ANA_CTRL.bitfields.ADC_BM.getbit()
    assert inval == retval

    print('# hal subscriptable interface...')
    print('# hal[0]')
    print(hal[0])

    print('# hal[0][0]')
    print(hal[0][0])

    print("# hal['ADC_ANA_CTRL']")
    print(hal['ADC_ANA_CTRL'])

    print('# hal dict-based reg assignement')
    regval = 1
    hal['ADC_ANA_CTRL'] = regval
    assert hal.regs.ADC_ANA_CTRL.getreg() == regval

    print('# hal dict-based bitfield assignement')
    fieldval = 2
    hal['ADC_ANA_CTRL']['ADC_BM'] = fieldval
    assert hal.regs.ADC_ANA_CTRL.bitfields.ADC_BM.getbit() == fieldval

    print('# hal dict-based assignement in single register write')
    dither = 1
    chop = 1
    inv = 1
    hal['ADC_ANA_CTRL'] = {'DITHER_EN': dither, 'CHOP_EN': chop, 'INV_CLK': inv}
    assert hal.regs.ADC_ANA_CTRL.bitfields.DITHER_EN.getbit() == dither
    assert hal.regs.ADC_ANA_CTRL.bitfields.CHOP_EN.getbit() == chop
    assert hal.regs.ADC_ANA_CTRL.bitfields.INV_CLK.getbit() == inv

    print('# hal search_register')
    searchreg = 'ADC_ANA_CTRL'
    reglist = hal.search_register(searchreg)
    reg = reglist[0]
    print(reg)
    assert str(reg) == searchreg

    print('# hal search_bitfield')
    searchfield = 'ADC_BM'
    fieldlist = hal.search_bitfield(searchfield)
    print(fieldlist)
    field = fieldlist[0]
    print(field)
    assert searchfield in field

    print('# hal search_address')
    searchaddr = '0x4000702c'
    reg = hal.search_address(searchaddr)
    print(f'# Register {reg}')
    addr = hex(hal[reg].addr)
    print(f'# Address: {addr}')
    assert addr == searchaddr

    print('# hal search_address with mask')
    searchaddr = '0xF000702c'
    searchmask = '0x0FFFFFFF'
    reg = hal.search_address(searchaddr,mask=searchmask)
    print(f'# Register {reg}')
    addr = hex(hal[reg].addr)
    print(f'# Address: {addr}')
    assert (literal_eval(addr)       & literal_eval(searchmask) ==
            literal_eval(searchaddr) & literal_eval(searchmask) )
    reg = hal.search_address(searchaddr)
    assert reg==''

    print('# hal dump')
    dump1 = 'dump1.hkl'
    dump2 = 'dump2.hkl'
    hal.dump(dump1)
    hal['ADC_ANA_CTRL']['ADC_BM'] = 3
    hal.dump(dump2)
    diff = hal.dump_diff(dump1,dump2)
    print(diff)
    assert len(diff) == 2

    print('# hal regs2dict')
    mydict = hal.regs2dict()
    assert isinstance(mydict,dict)

    print('# CpHalSuper inheritance')
    hal_super = CpHalSuper(hal)
    hal_super.super_method()

if __name__ == '__main__':
    test_cp_hal()



#!/usr/bin/python3
"""
bitfield register class for cheap-pie.
"""
#
# Autogenerated with SMOP
## this file is part of cheap_pie, a python tool for chip validation
## author: Marco Merlin
## email: marcomerli@gmail.com

from ast import literal_eval

class CpBitfield():  # pylint: disable=R0902
    """A bitfield register class """
    # field width
    width = 1

    # LSB
    lsb = 1

    # mask
    mask = 1

    # comments
    comments = ""

    # Address (Duplicate information)
    addr = 0

    # reg name
    regname = ""

    # field name
    fieldname = ""

    # host interface handler
    hif = None

    # read/write
    read_write = "rw"

    # reset value
    reset = 0

    def __init__(self, regfield="",regaddr=0, regname="", width="1", # pylint: disable=R0913
                 bit_offset="0", comments="", hif=None, read_write = "rw", reset=0):

        if isinstance(width,str):
            width= literal_eval(width)
        # field width
        self.width = int( width )

        if isinstance(bit_offset,str):
            lsb=literal_eval(bit_offset)
        else:
            lsb = bit_offset
        self.lsb = int( lsb )

        if lsb is None:
            print(f'Bad definition for register field {regname} @ {regfield}!')
            lsb=0

        if isinstance(regaddr,str):
            regaddr=literal_eval(regaddr)

        # mask
        bstr = "0b" + ("1" * self.width) + ("0" * self.lsb)
        self.mask=literal_eval( bstr )
        self.comments = comments
        self.addr = int( regaddr )
        self.regname = regname
        self.fieldname = regfield
        self.hif = hif

        assert isinstance(read_write,str)
        assert read_write.lower() in ['r','w','rw']
        self.read_write = read_write

        self.reset = reset

    def __str__(self,fieldval=None,width=25):
        """ returns value string of a bitfield from a register value
        input : regval value of the full register either in decimal or
        hexadecimal """
        #
        bitstr = self.fieldname  + ' [' + str(self.width) + ']'
        if not fieldval is None:
            bitstr += ' = ' + hex(fieldval)

        if self.width > 1:
            msb = str( self.lsb + self.width - 1)
            regstr = self.regname + '[' + msb + ':' + str(self.lsb) + ']'
        else:
            regstr = self.regname + '[' + str(self.lsb) + ']'

        fmtstr = '%%%ds @ %%%ds' % (width,width) # pylint: disable=C0209
        return fmtstr % (regstr,bitstr)

    def __repr__(self, regval = None):
        """
        displays position of a bitfield in a register
        """
        #
        if regval is None:
            fieldval = None
        else:
            fieldval = self.getbit(regval=regval)
        return self.__str__(fieldval=fieldval)

    def display(self,regval=None):
        """
        Display the bitfield
        """
        fieldval = self.getbit(regval=regval)
        fieldstr = self.__str__(fieldval=fieldval) # pylint: disable=C2801
        print(fieldstr)

    def getbit(self,regval=None,echo=False,as_signed=False): # ,*args,**kwargs):
        """ function display(self,regval=None,echo=False,as_signed=False)
        # displays value of a bitfield from a register value
        # input : regval value of the full register either in decimal or
        # hexadecimal"""
        #
        if regval is None:
            if self.hif is None:
                regval=0
            else:
                regval=self.hif.hifread(self.addr) # ,*args,**kwargs)

        # compute field value from register value
        if isinstance(regval, str):
            regval=literal_eval(regval)
        fieldval = (regval & self.mask ) >> (self.lsb)

        # get bitfield as signed value
        if as_signed:
            fieldsign = -(fieldval & (1 << (self.width - 1)))
            fieldmod = (fieldval & (self.mask>>(self.lsb+1)))
            fieldval = fieldsign + fieldmod

        if echo:
            self.display(regval)
        #
        return fieldval

    def setbit(self,fieldval=0,echo=False, writeback=True, regval=None,*args,**kwargs): # pylint: disable=W1113
        """ function display(self,regval)
        # displays value of a bitfield from a register value
        # input : regval value of the full register either in decimal or
        # hexadecimal """

        ## read input register value ###################################################
        if not (self.hif is None) and regval is None:
            hexval=self.hif.hifread(self.addr)
            if isinstance(hexval, str):
                regval=literal_eval(hexval)
            else:
                regval = hexval
        #
        ## handle char input as binary #################################################
        if isinstance(fieldval, str):
            fieldval = literal_eval(fieldval)

        ## handle negative values ######################################################
        if fieldval < 0:
            fieldval = (abs(fieldval) ^ (self.mask >> self.lsb)) + 1

        ## compute new register value ##################################################
        #
        shiftval= fieldval << self.lsb
        maskinv= self.mask ^ literal_eval('0xFFFFFFFF')
        regmasked = regval & maskinv
        outregval = regmasked + (shiftval & self.mask)

        # check new value in range
        if (shiftval & self.mask) < shiftval:
            raise ValueError(f'Bitifield value f{fieldval} out of range!')
        #
        ## write back new register value ###############################################
        if writeback:
            self.hif.hifwrite(self.addr,outregval,*args,**kwargs)
        #
        if echo:
            self.display(regval=regval)
        #
        return outregval

    #@function
    def help(self):
        """ function ret = help(self)
        # displays register comments """
        print(self.comments)

    def __index__(self):
        return int(self.getbit())

def test_cp_bitfield():
    """
    Test function for cp_bitfield class
    """
    import sys # pylint: disable=C0415
    import os.path # pylint: disable=C0415
    sys.path.append( os.path.join(os.path.dirname(__file__), '..') )
    from transport.cp_dummy_transport import CpDummyTransport # pylint: disable=C0415, disable=E0401

    field = CpBitfield(
        regfield = 'fname',
        regaddr = 10,
        regname = 'rname',
        width = '2',
        bit_offset = '2',
        comments = 'comment',
        hif = CpDummyTransport()
    )

    print('# setbit, getbit')
    val = 3
    field.setbit(val)
    assert field.getbit() == val

    print('# display')
    field.display(4)

    print('# str')
    print(str(field))

    print('# print')
    print(field)

    print('# help')
    field.help()

    print('# signed assignement')
    negval = -1
    field.setbit(negval)
    retval = field.getbit(as_signed=True)
    assert negval==retval

    print('# decimal representation')
    print(hex(field))

    print('# setbit with echo')
    field.setbit(val,echo=True)

    print('# setbit without writeback')
    val=1
    field.setbit(val)
    assert field.getbit() == val
    newval = 3
    field.setbit(newval,writeback=False)
    assert field.getbit() == val

    print('# setbit with regval')
    val=3
    regretval = field.setbit(val)
    assert field.setbit(val,regval=regretval) == regretval

    print('# test assertion')
    try:
        field.setbit(7)
        assert False, 'Assertion was not raised!!!'
    except ValueError as error:
        print(f'Assertion raised correctly: <{error}>')

if __name__ == '__main__':
    test_cp_bitfield()



#!/usr/bin/python3
""" Cheap Pie module wrapper around all parsers """
# -*- coding: utf-8 -*-
## this file is part of cheap_pie, a python tool for chip validation
## author: Marco Merlin
## email: marcomerli@gmail.com

import os

from cheap_pie.cheap_pie_core.cp_cli import cp_cli          # pylint: disable=C0415,E0401,C0413
from cheap_pie.parsers.svd_parse_repo import svd_parse_repo # pylint: disable=C0415,E0401,C0413
from cheap_pie.parsers.svd_parse import svd_parse           # pylint: disable=C0415,E0401,C0413
from cheap_pie.parsers.ipxact_parse import ipxact_parse     # pylint: disable=C0415,E0401,C0413
from cheap_pie.parsers.ipyxact_parse import ipyxact_parse   # pylint: disable=C0415,E0401,C0413
from cheap_pie.parsers.rdl_parse import rdl_parse           # pylint: disable=C0415,E0401,C0413

def cp_parsers_wrapper(prms,hif=None, base_address_offset = "0x00000000"):
    """ Cheap Pie function wrapper around all parsers """
    if prms.vendor is None:
        fname = os.path.join(prms.devicedir,prms.regfname)
        print(fname)
    else:
        # if vendor is indicated, use file from repository
        fname = prms.regfname
    if prms.format == 'cmsis-svd':
        # parser build for CMSIS-SVD xml file format
        hal = svd_parse_repo(fname=fname,hif=hif,vendor=prms.vendor,
                        base_address_offset=base_address_offset)
    elif prms.format == 'svd':
        # parser build for CMSIS-SVD xml file format
        hal = svd_parse(fname=fname,hif=hif,vendor=prms.vendor,
                        base_address_offset=base_address_offset)
    elif prms.format == 'ipxact':
        hal = ipxact_parse(fname=fname,hif=hif,
                           base_address_offset=base_address_offset)
    elif prms.format == 'ipyxact':
        hal = ipyxact_parse(fname=fname,hif=hif,
                           base_address_offset=base_address_offset)
    elif prms.format == 'rdl':
        hal = rdl_parse(fname=fname,hif=hif,
                        base_address_offset=base_address_offset)
    else:
        assert False, 'Unsupported input format!'

    # assert isinstance(hal,CpHal), f'Wrong type! {type(hal)}'
    return hal

def test_cp_parsers_wrapper():
    """ Test function for parsers wrapper """
    # from cheap_pie_core.cp_cli import cp_cli # pylint: disable=E0401,C0415
    prms = cp_cli()
    cp_parsers_wrapper(prms)

if __name__ == '__main__':
    test_cp_parsers_wrapper()



#!/usr/bin/python3

""" Convert between different .xml formats using .xslt rules
Examples of conversion between different IP-XACT versions
"""

import sys
import argparse
import os
import difflib

import lxml.etree as ET
from cheap_pie.cheap_pie_core.cp_cli import cp_devices_fname, cp_root_dir

def xml_xslt_parse(args=[]): # pylint: disable=W0102
    """ Parsing function for xml converter """
    parser = argparse.ArgumentParser(description='Apply XSLT to .xml file')
    parser.add_argument("-in", "--input", help=".xml input",
                        action='store', type = str, default=cp_devices_fname("generic_example.xml"))
    parser.add_argument("-xslt", "--xslt", help=".xslt file",
                        action='store', type = str,
                        default=os.path.join(
                            cp_root_dir(),
                            "parsers/ipxact_rules/from1685_2009_to_1685_2014.xsl"
                            )
                        )
    parser.add_argument("-out", "--output", help=".xml output",
                        action='store', type = str, default="output.xml")
    return parser.parse_args(args)

def xml_xslt(xmlin, xslt, xmlout):
    """ Convert between different .xml formats using .xslt rules """
    dom       = ET.parse(xmlin) # pylint: disable=I1101
    xslt      = ET.parse(xslt)  # pylint: disable=I1101
    transform = ET.XSLT(xslt)   # pylint: disable=I1101
    newdom    = transform(dom)
    infile    = ET.tostring(newdom, pretty_print=True).decode('utf-8') # pylint: disable=I1101
    # print(infile)
    if os.path.isfile(xmlout):
        os.remove(xmlout)
    with open(xmlout, 'a', encoding='utf-8') as outfile:
        outfile.write(infile)

def compare(fromfile,tofile):
    """ Compare two text files """
    with open(fromfile, encoding='utf-8') as ffh:
        fromlines = ffh.readlines()
    with open(tofile, encoding='utf-8') as tfh:
        tolines = tfh.readlines()
    diff = difflib.unified_diff(fromlines,tolines,fromfile=fromfile,tofile=tofile)

    difflines = []
    for line in diff:
        difflines.append(line)
        # print(line)

    assert len(difflines) > 0

def test_xml_xslt(args=[]): # pylint: disable=W0102
    """ Test convert between different .xml formats using .xslt rules """
    prms = xml_xslt_parse(args)
    xml_xslt(xmlin = prms.input, xslt = prms.xslt, xmlout = prms.output)
    # diff input output
    compare(prms.input,prms.output)

if __name__ == '__main__':
    test_xml_xslt(sys.argv[1:])



#!/usr/bin/python3

""" pyocd Cheap Pie transport module """

# -*- coding: utf-8 -*-
## this file is part of cheap_pie, a python tool for chip validation
## author: Marco Merlin
## email: marcomerli@gmail.com

from pyocd.core.helpers import ConnectHelper
from cheap_pie.transport.cp_dummy_transport import CpDummyTransport, test_cp_dummy, \
    hifread_preproc,hifwrite_preproc # pylint: disable=E0401
class CpPyocdTransport(CpDummyTransport):
    """ A wrapper around pyocd transport """
    ocd = None

    def __init__(self,device = None ):
        if not device is None:
            print('Connecting to pyocd probe... ')
            self.ocd = ConnectHelper.session_with_chosen_probe(target_override=device)
            self.ocd.open()
            # print(self.ocd.board)
            # print(self.ocd.target)
            # print(self.ocd.target.memory_map)

    def hifread(self, addr = "0x40000888"):
        """ read register through pyocd """

        addr = hifread_preproc(addr)

        if self.ocd is None:
            ret = CpDummyTransport.hifread(self,addr)
        else:
            # ret = self.ocd.board.target.dp.read_reg(addr)
            ret = self.ocd.board.target.read32(addr)

        return ret

    def hifwrite(self,addr = "0x40000888",val = "0x00000352"):
        """ rwrite register through pyocd """
        addr,val,_ = hifwrite_preproc(addr,val)

        if self.ocd is None:
            CpDummyTransport.hifwrite(self, addr, val)
        else:
            # ret = self.ocd.board.target.dp.write_reg(addr,val)
            self.ocd.board.target.write32(addr,val)

        return int( val )

def test_cp_pyocd():
    """ test pyocd transport """
    test_cp_dummy(
        CpPyocdTransport()
    )

if __name__ == '__main__':
    test_cp_pyocd()



#!/usr/bin/python3

""" JLink Cheap Pie transport module """

# -*- coding: utf-8 -*-
## this file is part of cheap_pie, a python tool for chip validation
## author: Marco Merlin
## email: marcomerli@gmail.com

import pylink
from cheap_pie.transport.cp_dummy_transport import CpDummyTransport, test_cp_dummy, \
    hifread_preproc,hifwrite_preproc # pylint: disable=E0401
class CpJlinkTransport(CpDummyTransport):
    """ A wrapper around jlink transport """
    jl = None

    def __init__(self, device = None ):
        if not device is None:
            self.jl = pylink.JLink() # pylint: disable=C0103
            self.jl.exec_command('ProjectFile = JLinkSettings.ini')
            self.jl.open()
            self.jl.set_speed(10)
            self.jl.set_tif(pylink.enums.JLinkInterfaces.SWD)
            self.jl.connect(device)
            assert self.jl.connected()

    def __del__(self):
        if not self.jl is None:
            self.jl.close()

    def hifread(self, addr = "0x40000888"):
        """ read register through JLink """

        addr = hifread_preproc(addr)

        if self.jl is None:
            ret = CpDummyTransport.hifread(self,addr)
        else:
            read = self.jl.memory_read32(addr,1)
            ret = read[0]

        return ret

    def hifwrite(self,addr = "0x40000888",val = "0x00000352"):
        """ write register through JLink """

        addr,val,_ = hifwrite_preproc(addr,val)

        if self.jl is None:
            CpDummyTransport.hifwrite(self, addr, val)
        else:
            self.jl.memory_write32( addr, [val] )

        return int( val )

def test_cp_jlink():
    """ test JLink transport """
    test_cp_dummy(
        CpJlinkTransport(device = None)
    )

if __name__ == '__main__':
    test_cp_jlink()



#!/usr/bin/python3

""" Cheap Pie Module to export register descriptio into .docx file """

## this file is part of cheap_pie, a python tool for chip validation
## author: Marco Merlin
## email: marcomerli@gmail.com

from docx import Document
# from docx.enum.table import WD_TABLE_ALIGNMENT
INCH2EMU = 914400

def int2hexstr(num,width): # pylint: disable=W0613
    """ Convert integer into hex string """
    fmt = "%%0%dx" % width          # pylint: disable=C0209
    # print fmt
    string = "\"%s\" %% num " % fmt # pylint: disable=C0209
    # print string
    return eval(string) # pylint: disable=W0123

def doc_create_header(template=None):
    """ Create document header """
    if template is None:
        document = Document()
    else :
        document = Document(template)

    # document.add_heading('Document Title', 0)
    document.add_heading('Register Description', level=1)
    document.add_paragraph("")

    return document

def doc_add_regtable(doc,reg,tablestyle=None,nbits_addr=32): # pylint: disable=R0912
    """ Add table description of a register to a document """
    # header
    doc.add_heading(reg.regname, level=3)

    # address
    doc.add_paragraph('Offset Address: 0x%s' % int2hexstr(reg.addr,nbits_addr/4)) # pylint: disable=C0209

    # description
    if isinstance(reg.comments,str):
        doc.add_paragraph(reg.comments)
    else:
        print("Warning: Non-string comments for reg: " + reg.regname)

    # bitfields table
    table = doc.add_table(rows=1, cols=5)

    if tablestyle is not None:
        table.style = tablestyle
    # table.alignment = WD_TABLE_ALIGNMENT.CENTER

    # configure column width
    # https://stackoverflow.com/questions/43749177/python-docx-table-column-width
    # inch2emu = 914400
    cwidths = [         1.5,  0.5,    0.5,      1,            3]

    # create table header
    headers = ['Field name','rwu','Bit #','Reset','Description']
    hdr_cells = table.rows[0].cells

    for idx in range(len(headers)): # pylint: disable=C0200
        hdr_cells[idx].text  = headers[idx]
        hdr_cells[idx].width = cwidths[idx]*INCH2EMU
        hdr_cells[idx].bold  = True
        # hdr_cells[idx].alignment=WD_ALIGN_PARAGRAPH.CENTER

    # add bitfields
    for field in reversed(reg.get_ordered_bitfields()):
        row_cells = table.add_row().cells
        row_cells[0].text = field.fieldname
        row_cells[0].width = cwidths[0]*INCH2EMU

        if len(field.read_write)/field.width > 1:
            row_cells[1].text = str.lower(field.read_write[0:2])
        else:
            row_cells[1].text = str.lower(field.read_write)
        row_cells[1].width = cwidths[1]*INCH2EMU

        if field.width == 1:
            row_cells[2].text = "%d" % ( field.lsb ) # pylint: disable=C0209
        else:
            row_cells[2].text = "%d:%d" % ( field.lsb + field.width -1, field.lsb) # pylint: disable=C0209
        row_cells[2].width = cwidths[2]*INCH2EMU

        if isinstance(field.reset,str):
            row_cells[3].text = field.reset
        else:
            resetstr = "%d\'%s" % (field.width, bin(field.reset)[1:]) # pylint: disable=C0209
            row_cells[3].text = resetstr
        row_cells[3].width = cwidths[3]*INCH2EMU

        if isinstance(reg.comments,str):
            row_cells[4].text = field.comments
            row_cells[4].width = cwidths[4]*INCH2EMU
            # print(field.comments)
        else:
            print(f"Warning: Non-string comments for field: \
                  {field.fieldname} in reg: {reg.regname}")

    doc.add_paragraph("")

def hal2doc(hal,fname='hal.docx',template=None,tablestyle=None,nbits_addr=32):
    """  export register description to .docx file """
    # initialize doc
    doc = doc_create_header(template)

    # loop over registers
    for reg in hal:
        # create register table
        doc_add_regtable(doc,reg,tablestyle,nbits_addr)

    # save document
    doc.save(fname)

def test_hal2doc():
    """ Test function for export tool from register description to .docx """
    print('Testing hal2doc...')

    from parsers.ipxact_parse import ipxact_parse      # pylint: disable=C0413,E0401,C0415
    from cheap_pie_core.cp_cli import cp_devices_fname # pylint: disable=C0413,E0401,C0415

    fname = cp_devices_fname("my_subblock.xml")
    hal = ipxact_parse(fname=fname)

    # convert to .docx
    hal2doc(hal)

if __name__ == '__main__':
    test_hal2doc()



#!/usr/bin/python3

""" Module including functions to search registers and bitifield in a
Cheap Pie register description
"""

## this file is part of cheap_pie, a python tool for chip validation
## author: Marco Merlin
## email: marcomerli@gmail.com

from ast import literal_eval

def str_in_str(str1,str2,case_sensitive=True):
    """ Check if a string is present into another string """
    if case_sensitive:
        return str1 in str2
    return str1.upper() in str2.upper()

def register(hal,regname,case_sensitive=True):
    """ Search a register by name """
    retval = []
    for reg in hal: # loop over all registers
        if str_in_str(regname,reg.regname,case_sensitive):
            print( reg.regname )
            retval.append(reg.regname)
    return retval

def bitfield(hal,bitfield_name,case_sensitive=True):
    """ Search a bitfield by name """
    retval = []
    for reg in hal: # loop over all registers
        # print reg.regname
        for field in reg.bitfields:
            if str_in_str(bitfield_name,field.fieldname,case_sensitive):
                fieldstr = str(field)
                print(fieldstr)
                retval.append(fieldstr)
    return retval

def address(hal, addr, mask='0xFFFFFFFF'):
    """ Search a register by address """
    # convert address into integer, if needed
    if isinstance(mask,str):
        mask    = int( literal_eval(mask) )
    if isinstance(addr,str):
        addr    = int( literal_eval(addr) )

    for reg in hal: # loop over all registers
        # print reg.regname
        if (reg.addr & mask) == (addr & mask):
            print( reg.regname + " : " + hex(reg.addr) )
            return reg.regname

    return ''

def test_search():
    """ Test Search Module """
    from parsers.svd_parse_repo import svd_parse_repo  # pylint: disable=E0401,C0415
    from cheap_pie_core.cp_cli import cp_devices_fname # pylint: disable=E0401,C0415
    print('Testing search...')

    fname = cp_devices_fname("QN908XC.svd")
    hal = svd_parse_repo(fname=fname, hif=None)

    print('## ADC registers:')
    ret = register(hal,'ADC')
    assert len(ret) > 0

    print('##  ADC_BM bitfields:')
    ret = bitfield(hal,'ADC_BM')
    assert len(ret) > 0

    print('## 0x4000702c register name:')
    ret = address(hal,'0x4000702c')
    assert len(ret) > 0

    print('## 0xF000702c register name:')
    ret = address(hal,'0xF000702c',mask='0x0FFFFFFF')
    assert len(ret) > 0

    ret = address(hal,'0xF000702c')
    assert ret==''

if __name__ == '__main__':
    import sys
    import os.path
    sys.path.append( os.path.join(os.path.dirname(__file__), '..') )
    test_search()



#!/usr/bin/python3

""" Esptool Cheap Pie transport module """

# -*- coding: utf-8 -*-
## this file is part of cheap_pie, a python tool for chip validation
## author: Marco Merlin
## email: marcomerli@gmail.com

import os
from ast import literal_eval
import esptool

from cheap_pie.transport.cp_dummy_transport import CpDummyTransport, test_cp_dummy # pylint: disable=E0401

class CpEsptoolTransport(CpDummyTransport):
    """ A wrapper around esptool transport """
    port = None

    def __init__(self, port='/dev/ttyUSB0' ):
        self.port = port

    def hifread(self, addr = '0x3ff00014'):
        """ read register through esptool """

        if isinstance(addr,int):
            # convert to string
            addr = hex(addr)

        if self.port is None:
            # fallback to dummy transport
            ret = CpDummyTransport.hifread(self,addr)
        else:
            #  ret = esptool.main( ['--port' , self.port , '--after no_reset', 'read_mem', addr]  )
            # dump value on file... horrible hack because no output available
            tmpfile = 'tmpdump'
            ret = esptool.main( # pylint: disable=E1101
                ['--port' , self.port , '--after',
                'no_reset', 'dump_mem', addr, '4', tmpfile]
                               )

            if os.path.isfile(tmpfile):
                with open(tmpfile, "rb") as tmpfh:
                    ret = '0x' + tmpfh.read().hex()
                os.remove(tmpfile)
            else:
                ret = 0

        return ret

    def hifwrite(self, addr = '0x3ff00014', val = "0x00000352"):
        """ write register through esptool """

        if isinstance(addr,int):
            addr = hex(addr)

        if isinstance(val, int):
            val = hex(val)

        if self.port is None:
            # fallback to dummy transport
            CpDummyTransport.hifwrite(self,addr,val)
        else:
            esptool.main( # pylint: disable=E1101
                ['--port' , self.port , '--after', 'no_reset' ,'write_mem', addr, val ,'0x0']
                )

        return literal_eval( val )

def test_cp_esptool():
    """ Test esptool transport """
    test_cp_dummy(
        transport = CpEsptoolTransport(port = None)
        )

if __name__ == '__main__':
    test_cp_esptool()



""" Transport Layers for Cheap Pie """
from .cp_dummy_transport import *
from .cp_jlink_transport import *
from .cp_pyocd_transport import *
from .cp_esptool_transport import *
from .cp_pyverilator_transport import *



""" Cheap Pie, a register and bitfield level tool for chip verification and validation """
from .cheap_pie_core import *
from .parsers import *
from .tools import *
from .transport import cp_dummy_transport



#!/usr/bin/python3
""" Cheap Pie parser module for IP-XACT structure with ipyxact """

## this file is part of cheap_pie, a python tool for chip validation
## author: Marco Merlin
## email: marcomerli@gmail.com

import sys
from ast import literal_eval
from ipyxact.ipyxact import Component              # pylint: disable=E0611

from cheap_pie.cheap_pie_core.cp_builder import CpHalBuilder # pylint: disable=C0413,E0401
from cheap_pie.cheap_pie_core.cp_cli     import cp_devices_fname # pylint: disable=C0413,E0401

def ipyxact_parse(fname,hif=None, base_address_offset = "0x00000000"):
    """ Cheap Pie parser for IP-XACT structure with ipyxact """
    ## read input file ########################################################
    xml = Component()
    xml.load(fname)

    ## loop over lines ########################################################
    cpb = CpHalBuilder(hif)

    for mem in xml.memoryMaps.memoryMap: # pylint: disable=R1702
        for periph in mem.addressBlock:
            if hasattr(periph,'register'):
                for reg in periph.register:
                    # new register
                    regaddr=(
                        reg.addressOffset +
                        periph.baseAddress +
                        literal_eval(base_address_offset)
                    )

                    cpb.reg_open(
                        regname=f'{periph.name}_{reg.name}',
                        regaddr=regaddr,
                        comments=reg.description,
                        )

                    if hasattr(reg,'field'):
                        for field in reg.field :
                            if not field is None:
                                # Create new field class
                                cpb.newfield(
                                    regfield = field.name,
                                    width = field.bitWidth,
                                    offset=field.bitOffset,
                                    comments=field.description
                                    )

    # convert output dictionary into structure
    return cpb.out()

def test_ipyxact_parse():
    """ Test function for IP-XACT parser with ipyxact """
    ipyxact_parse(cp_devices_fname("my_subblock.xml"))
    ipyxact_parse(cp_devices_fname("generic_example.xml"))

    # ipyxact_parse("./devices/leon2_creg.xml")
    # ValueError: invalid literal for int() with base 10: '4 * (2 ** 10)'

if __name__ == '__main__':
    if len(sys.argv) > 1:
        FNAME=sys.argv[1]
    else:
        FNAME=cp_devices_fname("my_subblock.xml")
    print(FNAME)
    hal = ipyxact_parse(FNAME)
    print(hal)
